==> reuse-4.3.4.tgz: reuse/source/win32/tempfile.c <==
/* $Id: tempfile.c 369 2004-09-11 06:13:25Z cher $ */

/* Copyright (C) 1999-2004 Alexander Chernov <cher@ispras.ru> */
/* Created: <1999-07-20 23:57:56 cher> */

/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 */

#define __REUSE__ 1

/* internal include directives */
#include "lconfig.h"
#include "reuse/config.h"
#include "embed_version.h"

/* reuse include directives */
#include <reuse/tempfile.h>
#include <reuse/logger.h>
#include <reuse/xalloc.h>
#include <reuse/osdeps.h>
#include <reuse/format_io.h>

#include <windows.h>
#include <limits.h>
#include <stdio.h>

struct tempnode_t
{
  int   used;
  char *path;
};
struct temparr_t
{
  int                a, u;
  struct tempnode_t *v;
};
static struct temparr_t temps;

static void logged_unlink(char const *);

  char *
temp_Create(const char *dir, const char *pfx, const char *sfx)
{
  int        i;
  int        num;
  SYSTEMTIME curtime;
  int        err;
  char       tempdir[1024];
  char       namebuf[1024];
  char       buf2[1024];
  HANDLE     h;

  for (i = 0; i < temps.u; i++) {
    if (!temps.v[i].used) break;
  }
  if (i >= temps.u) {
    XEXPAND2(temps);
    i = temps.u++;
  }

  if (dir) {
    strcpy(tempdir, dir);
  } else {
    fflush(stderr);
    GetTempPath(PATH_MAX, buf2);
    memcpy(tempdir, buf2, sizeof(buf2));
    tempdir[PATH_MAX - 1] = 0;
    i = strlen(tempdir);
    // mystery: if you remove the following fprintf
    // the whole `temp_Create' stops working... :-(
    //fprintf(stderr, "%s", "");
    if (i > 0) {
      i--;
      if (tempdir[i] == '\\' || tempdir[i] == '/') {
	tempdir[i] = 0;
      }
    }
  }
  if (!pfx) pfx = "";
  if (!sfx) sfx = ".TMP";

  GetSystemTime(&curtime);
  num = curtime.wMilliseconds;
  while (1) {
    os_snprintf(namebuf, PATH_MAX, "%s\\%s%d%s", tempdir, pfx, num, sfx);
    namebuf[PATH_MAX - 1] = 0;
    write_log(LOG_REUSE, LOG_INFO, "Temp name: '%s'", namebuf);

    h = CreateFile(namebuf, GENERIC_WRITE, 0, NULL, CREATE_NEW,
		   FILE_ATTRIBUTE_NORMAL, NULL);
    if (h != INVALID_HANDLE_VALUE) break;
    err = GetLastError();

    if (err != ERROR_ALREADY_EXISTS && err != ERROR_FILE_EXISTS) {
      write_log(LOG_REUSE, LOG_ERROR, "CreateFile failed: %d", err);
      return NULL;
    }

    num++;
  }
  CloseHandle(h);

  temps.v[i].used = 1;
  temps.v[i].path = xstrdup(namebuf);
  return temps.v[i].path;
}

  void
temp_Remove(const char *path)
{ 
  int i;

  if (!path) return;

  for (i = 0; i < temps.u; i++)
    if (temps.v[i].used && !strcmp(temps.v[i].path, path))
      break;

  if (i >= temps.u) return;

  logged_unlink(temps.v[i].path);

  xfree(temps.v[i].path);
  temps.v[i].used = 0;
  temps.v[i].path = 0;
}

  void
temp_Initialize(void)
{
}

  void
temp_Finalize(void)
{
  int i;

  for (i = 0; i < temps.u; i++)
    {
      if (!temps.v[i].used) continue;

      logged_unlink(temps.v[i].path);

      xfree(temps.v[i].path);
      temps.v[i].used = 0;
      temps.v[i].path = 0;
    }
}

  static void
logged_unlink(char const *path)
{
  if (!DeleteFile(path) < 0) {
      write_log(LOG_REUSE, LOG_ERROR,
                "Failed to remove temp file '%s': %d", path, GetLastError());
  } else {
    write_log(LOG_REUSE, LOG_INFO, "Temp file '%s' removed", path);
  }
}

#if REUSE_COMPILE_CVSID - 0 != 0
static char const cvsid[]=
"$Id: tempfile.c 369 2004-09-11 06:13:25Z cher $";
#endif /* REUSE_COMPILE_CVSID */

/*
 * Local variables:
 *  compile-command: "make -C .."
 *  c-font-lock-extra-types: ("HANDLE" "FILE" "\\sw+_t" "SYSTEMTIME")
 * End:
 */

==> reuse-4.3.9.tgz: reuse/source/win32/tempfile.c <==
/* $Id: tempfile.c 544 2009-12-14 08:39:39Z cher $ */

/* Copyright (C) 1999-2004 Alexander Chernov <cher@ispras.ru> */
/* Created: <1999-07-20 23:57:56 cher> */

/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 */

#define __REUSE__ 1

/* internal include directives */
#include "lconfig.h"
#include "reuse/config.h"
#include "embed_version.h"

/* reuse include directives */
#include <reuse/tempfile.h>
#include <reuse/logger.h>
#include <reuse/xalloc.h>
#include <reuse/osdeps.h>
#include <reuse/format_io.h>
#include <reuse/str_utils.h>

#include <windows.h>
#include <limits.h>
#include <stdio.h>

struct tempnode_t
{
  int   used;
  char *path;
};
struct temparr_t
{
  int                a, u;
  struct tempnode_t *v;
};
static struct temparr_t temps;

static void logged_unlink(char const *);

  char *
temp_Create(const char *dir, const char *pfx, const char *sfx)
{
  int        _i, ind, len;
  int        num;
  SYSTEMTIME curtime;
  int        err;
  char       tempdir[1024];
  char       namebuf[1024];
  char       buf2[1024];
  HANDLE     h;

  for (ind = 0; ind < temps.u; ind++) {
    if (!temps.v[ind].used) break;
  }
  if (ind >= temps.u) {
    XEXPAND2(temps);
    ind = temps.u++;
    memset(&temps.v[ind], 0, sizeof(temps.v[0]));
  }

  if (dir) {
    strlcpy(tempdir, dir, sizeof(tempdir));
  } else {
    GetTempPath(sizeof(buf2), buf2);
    strlcpy(tempdir, buf2, sizeof(tempdir));
    len = strlen(tempdir);
    if (len > 0) {
      len--;
      if (tempdir[len] == '\\' || tempdir[len] == '/') {
	tempdir[len] = 0;
      }
    }
  }
  if (!pfx) pfx = "";
  if (!sfx) sfx = ".TMP";

  GetSystemTime(&curtime);
  num = curtime.wMilliseconds;
  while (1) {
    _snprintf(namebuf, sizeof(namebuf), "%s\\%s%d%s", tempdir, pfx, num, sfx);
    write_log(LOG_REUSE, LOG_INFO, "Temp name: '%s'", namebuf);

    h = CreateFile(namebuf, GENERIC_WRITE, 0, NULL, CREATE_NEW,
		   FILE_ATTRIBUTE_NORMAL, NULL);
    if (h != INVALID_HANDLE_VALUE) break;
    err = GetLastError();

    if (err != ERROR_ALREADY_EXISTS && err != ERROR_FILE_EXISTS) {
      write_log(LOG_REUSE, LOG_ERROR, "CreateFile failed: %d", err);
      return NULL;
    }

    num++;
  }
  CloseHandle(h);

  temps.v[ind].used = 1;
  temps.v[ind].path = xstrdup(namebuf);
  return temps.v[ind].path;
}

  void
temp_Remove(const char *path)
{ 
  int i;

  if (!path) return;

  for (i = 0; i < temps.u; i++)
    if (temps.v[i].used && !strcmp(temps.v[i].path, path))
      break;

  if (i >= temps.u) return;

  logged_unlink(temps.v[i].path);

  xfree(temps.v[i].path);
  temps.v[i].used = 0;
  temps.v[i].path = 0;
}

  void
temp_Initialize(void)
{
}

  void
temp_Finalize(void)
{
  int i;

  for (i = 0; i < temps.u; i++)
    {
      if (!temps.v[i].used) continue;

      logged_unlink(temps.v[i].path);

      xfree(temps.v[i].path);
      temps.v[i].used = 0;
      temps.v[i].path = 0;
    }
}

  static void
logged_unlink(char const *path)
{
  if (!DeleteFile(path) < 0) {
      write_log(LOG_REUSE, LOG_ERROR,
                "Failed to remove temp file '%s': %d", path, GetLastError());
  } else {
    write_log(LOG_REUSE, LOG_INFO, "Temp file '%s' removed", path);
  }
}

#if REUSE_COMPILE_CVSID - 0 != 0
static char const cvsid[]=
"$Id: tempfile.c 544 2009-12-14 08:39:39Z cher $";
#endif /* REUSE_COMPILE_CVSID */

/*
 * Local variables:
 *  compile-command: "make -C .."
 *  c-font-lock-extra-types: ("HANDLE" "FILE" "\\sw+_t" "SYSTEMTIME")
 * End:
 */
